//********************************************************************
#include "colors.inc"
#include "stones.inc"
#include "shapes.inc"

#include "rail.inc" 
#include "HEAD2_POV_geom.inc"

#default {finish {ambient 0}}

// global_settings + photons added by William
global_settings {
   max_trace_level 20
   adc_bailout 1/127

   photons {
     count 20000
     max_trace_level 10
     autostop 0
     jitter 0
   }
 }

// end global_settings

// Begin additions by Yadgar
#declare s1=seed(1979);

#declare Yadgar_Silver = texture { //TA
   pigment { color rgb 0.75 }
   finish {
      brilliance 0.7
      diffuse 1
      reflection 0.5
      specular 0.8
      roughness 0.001
   }
};

#declare Silver_Ball= sphere {
    0, 0.04
    texture {Yadgar_Silver}
}
// End additions by Yadgar

camera {
   location <-5,10,-20>
//   location <2,3,-5>
   look_at 0
//   look_at <1,2,-1>

}

light_source {
   <1.5,1.3,-1.0>*10 color rgb 1.2 //brightened by Lukas Winter
   spotlight point_at y  radius 20 falloff 25 area_light  x*2,y*2,25,25
   circular    orient    adaptive 0 
   photons {
     refraction off reflection off
   }
 }

//Sam's light source changes
light_source {      // Separate non-area photon source
   <1.5,1.3,-1.011>*10 color rgb 1.2   
   spotlight point_at y  radius 20 falloff 25
   photons {
     refraction on reflection on
   }
 }  
  sphere {  <1.5,1.3,-1.011>*10, .1 photons{pass_through collect off } }     // Filter to let photons through, but not light. Explain that to a physicist.
// End light changes

//changes by Lukas Winter
#macro blurred_reflection(pigm, fin, amount, sampl)
  texture
  {
    average
    texture_map
    {
      #while(sampl > 0)
        [1
          pigment { pigm }
          finish { fin }
          normal
          {
            bumps amount
            rotate <rand(s1), rand(s1), rand(s1)>
            translate x*rand(s1)
            scale rand(s1)*.1
          }
        ]
        #declare sampl = sampl -1;
      #end
    }
  }
#end

#declare plane_pigment = pigment { Grey };
#declare plane_finish = finish { reflection .5 specular .2 };
plane
{
   y, 0
   blurred_reflection(plane_pigment,
     plane_finish,
     .4, 15)
   no_reflection
   photons
   {
     reflection off
     collect on
   }
}

//Aero's textures

#declare eyemirror = texture {
	pigment {color rgbt 1}
	finish {reflection 1}
}

#declare eyeglow = texture {
	pigment { color rgbt <0.9,0.1,0.1,0.8> }
	finish {
		reflection 0.1
		ambient 0
		diffuse 0.9
	}
}

plane
{
  y, 0
  pigment { plane_pigment }
  finish { plane_finish }
  no_image
  no_shadow
}


light_source
{
  <-8, .8, 3>
  color <1, 1, .8>
  fade_power 2
  fade_distance .8  
  shadowless
  photons {
    refraction off reflection off
  }
}
 
sphere
{
  <-8, .8, 3>, .8
  pigment
  {
    rgb
     <1, 1, 1>
  }      
  finish {ambient 2}  
  no_shadow
}


//end changes by Lukas Winter

#macro rubber(col)
   texture {
     pigment {color col}
     finish {specular 1/4}
     normal {bumps 1/50 scale 1/100}
   }
#end

sphere {
  <0,3,0>, 3 
  material{
          texture{
                  pigment{rgbt 1}
                  finish {reflection .01 specular .2}
          }
          interior {
                  ior 1.5
                  fade_color rgb <1, 0, 0>
                  fade_distance 3
          }
  }
  photons {
    target
        refraction on
        reflection on
        collect off
  }

}
// Sam's head in a ball

difference {
	object{
	      HEAD2_  rotate 155*y rotate 30*x scale .0625 translate <0, 3, -.5>  
	}
	sphere { 
		<0,3.5,-1>
		0.21
		texture { eyemirror }
	}
	sphere {
		<0.68,3.35,-0.74>
		0.21
		texture { eyemirror }
	}
    photons {
     target
     refraction on
     reflection on
     collect off
   }
}
      

// End of Sam's Head

// Aero's eyes

sphere { 
	<0,3.5,-1>
	0.20
	texture { eyeglow }
	photons {
		target
		reflection on
		refraction on
		collect off
	}
}

sphere {
	<0.68,3.35,-0.74>
	0.20
	texture { eyeglow }
	photons {
		target
		reflection on
		refraction on
		collect off
	}
}

difference {
	sphere {
		<0,3.5,-1>
		0.22
	}
	sphere {
		<0,3.5,-1.02>
		0.23
	}
	texture { eyemirror }
}

difference {
	sphere {
		<0.68,3.35,-0.74>
		0.22
	}
	sphere {
		<0.68,3.35,-0.76>
		0.23
	}
	texture { eyemirror }
}

light_source {
	<0,3.5,-0.8>
	color rgb <1.5,0,0>
  photons {
    refraction off reflection off
  }
}

light_source {
	<0.68,3.35,-0.54>
	color rgb <1.5,0,0>
  photons {
    refraction off reflection off
  } 
}

sphere {<4,2,-3>, 2 rubber(rgb x+y)}
sphere {<-2,1,-4>, 1  rubber(rgb z)}

// Box added by William
box {
   <0, 0, 0>
   <2, 2, 2>
   //Distorted by Alain
   rotate -21
   scale<1.3,1,0.8>
   rotate 21
   //end distortion
   rotate y * 45
   translate z * -5
   texture
   {
      pigment { color rgbt 0.75 }
      normal {bumps 0.5 scale 0.2}
      finish {
       brilliance 0.5
       diffuse 0.1
       reflection 0.5
       specular 0.75
       roughness 0.001
     }
     
   }
   interior
   {
     ior 1.35
   }
   photons {
     target
     refraction on
     reflection on
     collect off
   }
}
// End of William's box      


sphere{
  <1, 2, -05>, 0.75
  rubber(rgb <0, 0.5, 0>)
}


// Begin additions by Yadgar
#declare n=0;

#while (n<1800)
    #declare r1=rand(s1)*360;
    #declare r2=asin(rand(s1)*2-1);
    object { Silver_Ball translate <0, 3, 0>+3*<sin(r1*(pi/180))*cos(r2),
sin(r2), cos(r1*(pi/180))*cos(r2)> }
    #declare n=n+1;
#end

#declare n=0;

#while (n<800)
    #declare r1=rand(s1)*360;
    #declare r2=asin(rand(s1)*2-1);
    object { Silver_Ball translate <4, 2,
-3>+2*<sin(r1*(pi/180))*cos(r2), sin(r2), cos(r1*(pi/180))*cos(r2)> }
    #declare n=n+1;
#end

#declare n=0;

#while (n<200)
    #declare r1=rand(s1)*360;
    #declare r2=asin(rand(s1)*2-1);
    object { Silver_Ball translate <-2, 1, -4>+<sin(r1*(pi/180))*cos(r2),
sin(r2), cos(r1*(pi/180))*cos(r2)> }
    #declare n=n+1;
#end
// End additions by Yadgar

object {panelled_wall scale 1.5 rotate 30*y translate <2,-4,8>}


//Addition by Alain
/*union{
	plane{-z,-10 rotate -4*y}
	plane{z,-10 no_image no_shadow}
	pigment{rgb 0.9}
	hollow //to prevent the warning
	finish{reflection 0.97 }
	rotate 1*y
}
*/

// End addition by Alain

// Addition by Tim Attwood
#macro RingOf(Obj StartPos IncRot Num)
  #local _COUNT = 0;
  #local _ROT = <0,0,0>;
  #while (_COUNT < Num)
    object {Obj translate StartPos
      rotate <_ROT.x,0,0>
      rotate <0,_ROT.y,0>
      rotate <0,0,_ROT.z>}
    #local _ROT = _ROT + IncRot;
    #local _COUNT = _COUNT + 1;
  #end
#end

#declare pin = union{
   union {
    sphere{0,0.0625}
    difference{ sphere{0,0.0625 scale 2*x}plane{x,0}} texture {
      pigment{Violet}
      finish {reflection 0.1
      specular 0.3
      roughness 0.001}
    }
    }
   cylinder{0,1.4*x,0.015}
   cone{<1.4,0,0>,0.015,<1.825,0,0>,0}
   rotate <0,0,-1.9>
   scale 1
   translate <0,-0.965,0>
   texture{Yadgar_Silver}
}
union { RingOf(pin, <-2,0,0>,<0,7,0>,12) rotate -75*y translate <-3,1,-5>}
// End addition by Tim Attwood
